/** @file fifo_coverage.h
 * This file contains the coverage class declaration.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#pragma once

#include <hw/model.hpp>
#include <ts/coverage.hpp>
#include <tracer/tracer.hpp>

using namespace cpptesk::ts;
using namespace cpptesk::hw;
using namespace cpptesk::tracer;
using namespace cpptesk::tracer::coverage;

namespace cpptesk {
namespace examples {
namespace fifo {

/// Pre-declaration of the reference model
class FIFO;

/// Functional coverage based on the reference model
class FIFOCoverage {
public:
    FIFOCoverage(FIFO &fifo): fifo(fifo) {}

    /// This object tracks coverage
    CoverageTracker coverageTracker;

    /// Coverage "Fullness of the reference model FIFO"
    ENUM_COV(FIFO_FULLNESS, "FIFO fullness", (
                            (I0, "0 (free)"),
                            (I1, "1"),
                            (I2, "2"),
                            (I3, "3"),
                            (I4, "4"),
                            (I5, "5"),
                            (I6, "6"),
                            (I7, "7"),
                            (I8, "8"),
                            (I9, "9"),
                            (I10, "10"),
                            (I11, "11"),
                            (I12, "12"),
                            (I13, "13"),
                            (I14, "14"),
                            (I15, "15"),
                            (I16, "16 (full)")
            ));

    /// This function binds current reference model state and the FIFO_FILLNESS functional coverage
    FIFO_FULLNESS cov_FF(void) const;

    /// Function refreshing coverage
    void out_cov_FF(void);

protected:
    /// Link to the reference model
    FIFO &fifo;
};

}}}

