/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>

#include <hw/model.hpp>
#include <hw/message.hpp>

using namespace cpptesk::hw;

////////////////////////////////////////////////////////////////////////////////
// Message Model
////////////////////////////////////////////////////////////////////////////////
CPPTESK_MESSAGE(MSG)
{
public:
    MSG()
    {
        CPPTESK_ADD_FIELD(MSG::data);
    }
    
    virtual ~MSG() {}

    CPPTESK_SUPPORT_CLONE(MSG)

    CPPTESK_DECLARE_FIELD(data, 8);
};

////////////////////////////////////////////////////////////////////////////////
// Design Model
////////////////////////////////////////////////////////////////////////////////
CPPTESK_MODEL(DUT)
{
public:
////////////////////////////////////////////////////////////////////////////////
// Input interface
////////////////////////////////////////////////////////////////////////////////
    CPPTESK_DECLARE_IFACE(iface);

////////////////////////////////////////////////////////////////////////////////
// Design Stimuli
////////////////////////////////////////////////////////////////////////////////

    CPPTESK_DECLARE_PROCESS(process2)
    {
        START_PROCESS();

	CPPTESK_DEBUG_PRINT(DEBUG_ERROR, "Bug has been found");

        STOP_PROCESS();
    }

    CPPTESK_DECLARE_PROCESS(process1)
    {
        START_PROCESS();

        CALL_PROCESS(PARALLEL, process2, GET_IFACE(), GET_MESSAGE());

        CPPTESK_DEBUG_PRINTF(DEBUG_USER, "[%03d]: thread: stage 0", process.pid);
        CYCLE();

        CPPTESK_DEBUG_PRINTF(DEBUG_USER, "[%03d]: thread: stage 1", process.pid);
        CYCLE();

        STOP_PROCESS();
    }

    CPPTESK_DECLARE_STIMULUS(stimulus)
    {
        int i;
        START_STIMULUS(SEQUENTIAL);

        // Create a new process
        CALL_PROCESS(SEQUENTIAL, process1, GET_IFACE(), GET_MESSAGE());

        // Execute the code sequentially within the given process
        CALL_PROCESS(PARALLEL, process1, GET_IFACE(), GET_MESSAGE());

	// Just call it as a function
	process1(GET_PARAMETERS());

        for(i = 0; i < 9; i++)
        {
            CPPTESK_DEBUG_PRINTF(DEBUG_USER, "[%03d]: stage=%d", process.pid, i);
            CYCLE();
        }

        CPPTESK_DEBUG_PRINTF(DEBUG_USER, "[%03d]: stage=%d", process.pid, i);

	SEND_REACTION(PARALLEL, GET_IFACE(), GET_MESSAGE());

        STOP_STIMULUS();
    }

////////////////////////////////////////////////////////////////////////////////
// Design Mediators
////////////////////////////////////////////////////////////////////////////////
    virtual void onEveryCycle()
    {
        CPPTESK_DEBUG_PRINT(DEBUG_USER, "onEveryCycle: time=" << std::dec << time());
    }
};

////////////////////////////////////////////////////////////////////////////////
// Testbench
////////////////////////////////////////////////////////////////////////////////
int main()
{
    DUT dut;
    MSG msg;

    CPPTESK_SET_DEBUG_LEVEL(DEBUG_INFO, true);

    CPPTESK_SET_DEBUG_STYLE(DEBUG_USER, BLACK, CYAN, CYAN, BLACK);

    dut.initialize();

    for(int i = 0; i < 100; i++)
    {
        CPPTESK_CALL_PROCESS_OF(dut, DUT::stimulus, dut.iface, msg);
        dut.cycle();
    }

    for(int i = 0; i < 20; i++)
    {
        dut.cycle();
    }

    CPPTESK_DEBUG_PRINT(DEBUG_USER,  "CPPTESKHW_VERSION=" <<  CPPTESKHW_VERSION);
    CPPTESK_DEBUG_PRINT(DEBUG_ERROR, "This is the end");

    dut.finalize();

    return 0;
}

