/** @file fifo_media.h
 * This file contains the mediator class declaration.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#pragma once

#include <hw/veritool/media.hpp>
#include <fifo_model.h>
#include <interface.h>

#include <string>

namespace cpptesk {
namespace examples {
namespace fifo {

/**
 * @brief Adapter for FIFO reference model
 */
VERITOOL_ADAPTER(FIFOMediator, FIFO)
{
public:
    FIFOMediator();
    virtual ~FIFOMediator();

    /// Max timeout while waiting of registered reactions
    const static int REACTION_TIMEOUT = 100;

    virtual bool isIface1Ready() const;
    virtual bool isIface2Ready() const;

    /// Serializer of input iface1
    DECLARE_PROCESS(serialize_iface1);
    /// Serializer of input iface2
    DECLARE_PROCESS(serialize_iface2);

    /// Deserializer of output iface3
    DECLARE_PROCESS(deserialize_iface3);

    /// Detector of unexpected reactions on interface iface3
    DEFINE_BASIC_OUTPUT_LISTENER(listen_iface3, iface3, outputs.DO_VAL);

    /// Arbiter for interface iface3 (reaction flow splitter)
    DECLARE_ARBITER(PRIORITY_ARBITER, matcher_iface3);
};

}}}

